@@ -12,72 +12,18 @@ from os import listdir, lstat, walk
 from os.path import getsize, isdir, join
 import sys
 
-from .common import add_parser_json, add_parser_yes, confirm_yn, stdout_json
 from ..base.constants import CONDA_TARBALL_EXTENSION
 from ..base.context import context
 
 log = getLogger(__name__)
 
-descr = """
-Remove unused packages and caches.
-"""
-
-example = """
-Examples:
-
-    conda clean --tarballs
-"""
-
-def configure_parser(sub_parsers):
-    p = sub_parsers.add_parser(
-        'clean',
-        description=descr,
-        help=descr,
-        epilog=example,
-    )
-    add_parser_yes(p)
-    add_parser_json(p)
-    p.add_argument(
-        "-a", "--all",
-        action="store_true",
-        help="Remove index cache, lock files, tarballs, "
-             "unused cache packages, and source cache.",
-    )
-    p.add_argument(
-        "-i", "--index-cache",
-        action="store_true",
-        help="Remove index cache.",
-    )
-    p.add_argument(
-        "-l", "--lock",
-        action="store_true",
-        help="Remove all conda lock files.",
-    )
-    p.add_argument(
-        "-t", "--tarballs",
-        action="store_true",
-        help="Remove cached package tarballs.",
-    )
-    p.add_argument(
-        '-p', '--packages',
-        action='store_true',
-        help="""Remove unused cached packages. Warning: this does not check
-    for symlinked packages.""",
-    )
-    p.add_argument(
-        '-s', '--source-cache',
-        action='store_true',
-        help="""Remove files from the source cache of conda build.""",
-    )
-    p.set_defaults(func=execute)
-
 
 def find_tarballs():
-    from ..core.package_cache import PackageCache
+    from ..core.package_cache_data import PackageCacheData
     pkgs_dirs = defaultdict(list)
     totalsize = 0
     part_ext = CONDA_TARBALL_EXTENSION + '.part'
-    for package_cache in PackageCache.all_writable(context.pkgs_dirs):
+    for package_cache in PackageCacheData.writable_caches(context.pkgs_dirs):
         pkgs_dir = package_cache.pkgs_dir
         if not isdir(pkgs_dir):
             continue
@@ -91,6 +37,10 @@ def find_tarballs():
 
 
 def rm_tarballs(args, pkgs_dirs, totalsize, verbose=True):
+    from .common import confirm_yn
+    from ..gateways.disk.delete import rm_rf
+    from ..utils import human_bytes
+
     if verbose:
         for pkgs_dir in pkgs_dirs:
             print('Cache location: %s' % pkgs_dir)
@@ -101,7 +51,6 @@ def rm_tarballs(args, pkgs_dirs, totalsize, verbose=True):
         return
 
     if verbose:
-        from ..utils import human_bytes
         print("Will remove the following tarballs:")
         print()
 
@@ -118,14 +67,13 @@ def rm_tarballs(args, pkgs_dirs, totalsize, verbose=True):
         print()
 
     if not context.json or not context.always_yes:
-        confirm_yn(args)
+        confirm_yn()
     if context.json and args.dry_run:
         return
 
     for pkgs_dir in pkgs_dirs:
         for fn in pkgs_dirs[pkgs_dir]:
             try:
-                from ..gateways.disk.delete import rm_rf
                 if rm_rf(os.path.join(pkgs_dir, fn)):
                     if verbose:
                         print("Removed %s" % fn)
@@ -149,7 +97,8 @@ def find_pkgs():
     pkgs_dirs = defaultdict(list)
     for pkgs_dir in context.pkgs_dirs:
         if not os.path.exists(pkgs_dir):
-            print("WARNING: {0} does not exist".format(pkgs_dir))
+            if not context.json:
+                print("WARNING: {0} does not exist".format(pkgs_dir))
             continue
         pkgs = [i for i in listdir(pkgs_dir)
                 if (isdir(join(pkgs_dir, i)) and  # only include actual packages
@@ -190,11 +139,10 @@ def find_pkgs():
     return pkgs_dirs, warnings, totalsize, pkgsizes
 
 
-def rm_pkgs(args, pkgs_dirs, warnings, totalsize, pkgsizes,
-            verbose=True):
+def rm_pkgs(args, pkgs_dirs, warnings, totalsize, pkgsizes, verbose=True):
+    from .common import confirm_yn
     from ..gateways.disk.delete import rm_rf
     from ..utils import human_bytes
-
     if verbose:
         for pkgs_dir in pkgs_dirs:
             print('Cache location: %s' % pkgs_dir)
@@ -221,7 +169,7 @@ def rm_pkgs(args, pkgs_dirs, warnings, totalsize, pkgsizes,
         print()
 
     if not context.json or not context.always_yes:
-        confirm_yn(args)
+        confirm_yn()
     if context.json and args.dry_run:
         return
 
@@ -234,8 +182,8 @@ def rm_pkgs(args, pkgs_dirs, warnings, totalsize, pkgsizes,
 
 def rm_index_cache():
     from ..gateways.disk.delete import rm_rf
-    from ..core.package_cache import PackageCache
-    for package_cache in PackageCache.all_writable():
+    from ..core.package_cache_data import PackageCacheData
+    for package_cache in PackageCacheData.writable_caches():
         rm_rf(join(package_cache.pkgs_dir, 'cache'))
 
 
@@ -267,34 +215,38 @@ def find_source_cache():
 
 
 def rm_source_cache(args, cache_dirs, warnings, cache_sizes, total_size):
+    from .common import confirm_yn
     from ..gateways.disk.delete import rm_rf
     from ..utils import human_bytes
 
-    verbose = not context.json
+    verbose = not (context.json or context.quiet)
     if warnings:
-        if verbose:
+        if verbose:  # lgtm [py/uninitialized-local-variable]
             for warning in warnings:
                 print(warning, file=sys.stderr)
         return
 
-    for cache_type in cache_dirs:
-        print("%s (%s)" % (cache_type, cache_dirs[cache_type]))
-        print("%-40s %10s" % ("Size:", human_bytes(cache_sizes[cache_type])))
-        print()
+    if verbose:  # lgtm [py/uninitialized-local-variable]
+        for cache_type in cache_dirs:
+            print("%s (%s)" % (cache_type, cache_dirs[cache_type]))
+            print("%-40s %10s" % ("Size:", human_bytes(cache_sizes[cache_type])))
+            print()
 
-    print("%-40s %10s" % ("Total:", human_bytes(total_size)))
+        print("%-40s %10s" % ("Total:", human_bytes(total_size)))
 
     if not context.json or not context.always_yes:
-        confirm_yn(args)
+        confirm_yn()
     if context.json and args.dry_run:
         return
 
     for dir in cache_dirs.values():
-        print("Removing %s" % dir)
+        if verbose:
+            print("Removing %s" % dir)
         rm_rf(dir)
 
 
 def execute(args, parser):
+    from .common import stdout_json
     json_result = {
         'success': True
     }
@@ -308,7 +260,7 @@ def execute(args, parser):
             'files': pkgs_dirs[first],  # Backwards compatibility
             'total_size': totalsize
         }
-        rm_tarballs(args, pkgs_dirs, totalsize, verbose=not context.json)
+        rm_tarballs(args, pkgs_dirs, totalsize, verbose=not (context.json or context.quiet))
 
     if args.index_cache or args.all:
         json_result['index_cache'] = {
@@ -328,7 +280,7 @@ def execute(args, parser):
             'pkg_sizes': {i: dict(zip(pkgs_dirs[i], pkgsizes[i])) for i in pkgs_dirs},
         }
         rm_pkgs(args, pkgs_dirs,  warnings, totalsize, pkgsizes,
-                verbose=not context.json)
+                verbose=not (context.json or context.quiet))
 
     if args.source_cache or args.all:
         json_result['source_cache'] = find_source_cache()
