@@ -4,6 +4,7 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 from contextlib import contextmanager
 from logging import getLogger
 import signal
+import threading
 
 from .compat import iteritems
 
@@ -17,7 +18,6 @@ INTERRUPT_SIGNALS = (
     'SIGBREAK',
 )
 
-
 def get_signal_name(signum):
     """
     Examples:
@@ -33,18 +33,25 @@ def get_signal_name(signum):
 
 @contextmanager
 def signal_handler(handler):
-    previous_handlers = []
+    # TODO: test and fix windows
+    #   https://danielkaes.wordpress.com/2009/06/04/how-to-catch-kill-events-with-python/
+    _thread_local = threading.local()
+    _thread_local.previous_handlers = []
     for signame in INTERRUPT_SIGNALS:
         sig = getattr(signal, signame, None)
         if sig:
             log.debug("registering handler for %s", signame)
-            prev_handler = signal.signal(sig, handler)
-            previous_handlers.append((sig, prev_handler))
+            try:
+                prev_handler = signal.signal(sig, handler)
+                _thread_local.previous_handlers.append((sig, prev_handler))
+            except ValueError as e:  # pragma: no cover
+                # ValueError: signal only works in main thread
+                log.debug('%r', e)
     try:
         yield
     finally:
         standard_handlers = signal.SIG_IGN, signal.SIG_DFL
-        for sig, previous_handler in previous_handlers:
+        for sig, previous_handler in _thread_local.previous_handlers:
             if callable(previous_handler) or previous_handler in standard_handlers:
-                log.debug("de-registering handler for %s", signame)
+                log.debug("de-registering handler for %s", sig)
                 signal.signal(sig, previous_handler)
