@@ -1,146 +1,29 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+from copy import copy
 from itertools import chain
 from logging import getLogger
 
-from ..base.constants import (DEFAULTS_CHANNEL_NAME, MAX_CHANNEL_PRIORITY, UNKNOWN_CHANNEL)
+from .._vendor.boltons.setutils import IndexedSet
+from ..base.constants import DEFAULTS_CHANNEL_NAME, MAX_CHANNEL_PRIORITY, UNKNOWN_CHANNEL
 from ..base.context import context
 from ..common.compat import ensure_text_type, isiterable, iteritems, odict, with_metaclass
 from ..common.path import is_path, win_path_backout
 from ..common.url import (Url, has_scheme, is_url, join_url, path_to_url,
-                          split_conda_url_easy_parts, split_scheme_auth_token, urlparse)
+                          split_conda_url_easy_parts, split_platform, split_scheme_auth_token,
+                          urlparse)
 
 try:
     from cytoolz.functoolz import excepts
-    from cytoolz.itertoolz import concat, concatv, topk
-except ImportError:
+    from cytoolz.itertoolz import concat, concatv, drop
+except ImportError:  # pragma: no cover
     from .._vendor.toolz.functoolz import excepts  # NOQA
-    from .._vendor.toolz.itertoolz import concat, concatv, topk  # NOQA
+    from .._vendor.toolz.itertoolz import concat, concatv, drop  # NOQA
 
 log = getLogger(__name__)
 
 
-# backward compatibility for conda-build
-def get_conda_build_local_url():
-    return context.local_build_root,
-
-
-def tokenized_startswith(test_iterable, startswith_iterable):
-    return all(t == sw for t, sw in zip(test_iterable, startswith_iterable))
-
-
-def tokenized_conda_url_startswith(test_url, startswith_url):
-    test_url, startswith_url = urlparse(test_url), urlparse(startswith_url)
-    if test_url.host != startswith_url.host or test_url.port != startswith_url.port:
-        return False
-    norm_url_path = lambda url: url.path.strip('/') or '/'
-    return tokenized_startswith(norm_url_path(test_url).split('/'),
-                                norm_url_path(startswith_url).split('/'))
-
-
-def _get_channel_for_name(channel_name):
-    def _get_channel_for_name_helper(name):
-        if name in context.custom_channels:
-            return context.custom_channels[name]
-        else:
-            test_name = name.rsplit('/', 1)[0]  # progressively strip off path segments
-            if test_name == name:
-                return None
-            return _get_channel_for_name_helper(test_name)
-
-    channel = _get_channel_for_name_helper(channel_name)
-
-    if channel is not None:
-        # stripping off path threw information away from channel_name (i.e. any potential subname)
-        # channel.name *should still be* channel_name
-        channel.name = channel_name
-        return channel
-    else:
-        ca = context.channel_alias
-        return Channel(scheme=ca.scheme, auth=ca.auth, location=ca.location, token=ca.token,
-                       name=channel_name)
-
-
-def _read_channel_configuration(scheme, host, port, path):
-    # return location, name, scheme, auth, token
-
-    path = path and path.rstrip('/')
-    test_url = Url(host=host, port=port, path=path).url
-
-    # Step 1. No path given; channel name is None
-    if not path:
-        return Url(host=host, port=port).url.rstrip('/'), None, scheme or None, None, None
-
-    # Step 2. migrated_custom_channels matches
-    for name, location in sorted(context.migrated_custom_channels.items(), reverse=True,
-                                 key=lambda x: len(x[0])):
-        location, _scheme, _auth, _token = split_scheme_auth_token(location)
-        if tokenized_conda_url_startswith(test_url, join_url(location, name)):
-            # translate location to new location, with new credentials
-            subname = test_url.replace(join_url(location, name), '', 1).strip('/')
-            channel_name = join_url(name, subname)
-            channel = _get_channel_for_name(channel_name)
-            return channel.location, channel_name, channel.scheme, channel.auth, channel.token
-
-    # Step 3. migrated_channel_aliases matches
-    for migrated_alias in context.migrated_channel_aliases:
-        if test_url.startswith(migrated_alias.location):
-            name = test_url.replace(migrated_alias.location, '', 1).strip('/')
-            ca = context.channel_alias
-            return ca.location, name, ca.scheme, ca.auth, ca.token
-
-    # Step 4. custom_channels matches
-    for name, channel in sorted(context.custom_channels.items(), reverse=True,
-                                key=lambda x: len(x[0])):
-        that_test_url = join_url(channel.location, channel.name)
-        if tokenized_startswith(test_url.split('/'), that_test_url.split('/')):
-            subname = test_url.replace(that_test_url, '', 1).strip('/')
-            return (channel.location, join_url(channel.name, subname), scheme,
-                    channel.auth, channel.token)
-
-    # Step 5. channel_alias match
-    ca = context.channel_alias
-    if ca.location and tokenized_startswith(test_url.split('/'), ca.location.split('/')):
-        name = test_url.replace(ca.location, '', 1).strip('/') or None
-        return ca.location, name, scheme, ca.auth, ca.token
-
-    # Step 6. not-otherwise-specified file://-type urls
-    if host is None:
-        # this should probably only happen with a file:// type url
-        assert port is None
-        location, name = test_url.rsplit('/', 1)
-        if not location:
-            location = '/'
-        _scheme, _auth, _token = 'file', None, None
-        return location, name, _scheme, _auth, _token
-
-    # Step 7. fall through to host:port as channel_location and path as channel_name
-    return (Url(host=host, port=port).url.rstrip('/'), path.strip('/') or None,
-            scheme or None, None, None)
-
-
-def parse_conda_channel_url(url):
-    (scheme, auth, token, platform, package_filename,
-     host, port, path, query) = split_conda_url_easy_parts(url, context.known_subdirs)
-
-    # recombine host, port, path to get a channel_name and channel_location
-    (channel_location, channel_name, configured_scheme, configured_auth,
-     configured_token) = _read_channel_configuration(scheme, host, port, path)
-
-    # if we came out with no channel_location or channel_name, we need to figure it out
-    # from host, port, path
-    assert channel_location is not None or channel_name is not None
-
-    return Channel(configured_scheme or 'https',
-                   auth or configured_auth,
-                   channel_location,
-                   token or configured_token,
-                   channel_name,
-                   platform,
-                   package_filename)
-
-
 class ChannelType(type):
     """
     This metaclass does basic caching and enables static constructor method usage with a
@@ -158,7 +41,14 @@ class ChannelType(type):
                 c = Channel._cache_[value] = Channel.from_value(value)
                 return c
         else:
-            return super(ChannelType, cls).__call__(*args, **kwargs)
+            if 'channels' in kwargs:
+                # presence of 'channels' kwarg indicates MultiChannel
+                name = kwargs['name']
+                channels = tuple(super(ChannelType, cls).__call__(**_kwargs)
+                                 for _kwargs in kwargs['channels'])
+                return MultiChannel(name, channels)
+            else:
+                return super(ChannelType, cls).__call__(*args, **kwargs)
 
 
 @with_metaclass(ChannelType)
@@ -171,11 +61,11 @@ class Channel(object):
     channel <> subchannel <> namespace <> package_name
 
     """
-    _cache_ = dict()
+    _cache_ = {}
 
     @staticmethod
     def _reset_state():
-        Channel._cache_ = dict()
+        Channel._cache_ = {}
 
     def __init__(self, scheme=None, auth=None, location=None, token=None, name=None,
                  platform=None, package_filename=None):
@@ -195,6 +85,10 @@ class Channel(object):
     def channel_name(self):
         return self.name
 
+    @property
+    def subdir(self):
+        return self.platform
+
     @staticmethod
     def from_url(url):
         return parse_conda_channel_url(url)
@@ -220,9 +114,10 @@ class Channel(object):
             return Channel.from_url(value)
         else:
             # at this point assume we don't have a bare (non-scheme) url
-            #   e.g. this would be bad:  repo.continuum.io/pkgs/free
-            if value in context.custom_multichannels:
-                return MultiChannel(value, context.custom_multichannels[value])
+            #   e.g. this would be bad:  repo.anaconda.com/pkgs/free
+            _stripped, platform = split_platform(value, context.known_subdirs)
+            if _stripped in context.custom_multichannels:
+                return MultiChannel(_stripped, context.custom_multichannels[_stripped], platform)
             else:
                 return Channel.from_channel_name(value)
 
@@ -327,8 +222,23 @@ class Channel(object):
             return None
         return "%s://%s" % (self.scheme, join_url(self.location, self.name))
 
+    @property
+    def base_urls(self):
+        return self.base_url,
+
+    @property
+    def subdir_url(self):
+        url = self.url(True)
+        if self.package_filename and url:
+            url = url.rsplit('/', 1)[0]
+        return url
+
     def __str__(self):
-        return self.base_url or ""
+        base = self.base_url or self.name
+        if self.subdir:
+            return join_url(base, self.subdir)
+        else:
+            return base
 
     def __repr__(self):
         return ("Channel(scheme=%r, auth=%r, location=%r, token=%r, name=%r, platform=%r, "
@@ -344,7 +254,12 @@ class Channel(object):
         if isinstance(other, Channel):
             return self.location == other.location and self.name == other.name
         else:
-            return False
+            try:
+                _other = Channel(other)
+                return self.location == _other.location and self.name == _other.name
+            except Exception as e:
+                log.debug("%r", e)
+                return False
 
     def __hash__(self):
         return hash((self.location, self.name))
@@ -362,18 +277,35 @@ class Channel(object):
     def url_channel_wtf(self):
         return self.base_url, self.canonical_name
 
+    def dump(self):
+        return {
+            "scheme": self.scheme,
+            "auth": self.auth,
+            "location": self.location,
+            "token": self.token,
+            "name": self.name,
+            "platform": self.platform,
+            "package_filename": self.package_filename,
+        }
+
 
 class MultiChannel(Channel):
 
-    def __init__(self, name, channels):
+    def __init__(self, name, channels, platform=None):
         self.name = name
         self.location = None
-        self._channels = channels
+
+        if platform:
+            c_dicts = tuple(c.dump() for c in channels)
+            any(cd.update(platform=platform) for cd in c_dicts)
+            self._channels = tuple(Channel(**cd) for cd in c_dicts)
+        else:
+            self._channels = channels
 
         self.scheme = None
         self.auth = None
         self.token = None
-        self.platform = None
+        self.platform = platform
         self.package_filename = None
 
     @property
@@ -392,9 +324,149 @@ class MultiChannel(Channel):
     def base_url(self):
         return None
 
+    @property
+    def base_urls(self):
+        return tuple(c.base_url for c in self._channels)
+
     def url(self, with_credentials=False):
         return None
 
+    def dump(self):
+        return {
+            "name": self.name,
+            "channels": tuple(c.dump() for c in self._channels)
+        }
+
+
+def tokenized_startswith(test_iterable, startswith_iterable):
+    return all(t == sw for t, sw in zip(test_iterable, startswith_iterable))
+
+
+def tokenized_conda_url_startswith(test_url, startswith_url):
+    test_url, startswith_url = urlparse(test_url), urlparse(startswith_url)
+    if test_url.host != startswith_url.host or test_url.port != startswith_url.port:
+        return False
+    norm_url_path = lambda url: url.path.strip('/') or '/'
+    return tokenized_startswith(norm_url_path(test_url).split('/'),
+                                norm_url_path(startswith_url).split('/'))
+
+
+def _get_channel_for_name(channel_name):
+    def _get_channel_for_name_helper(name):
+        if name in context.custom_channels:
+            return context.custom_channels[name]
+        else:
+            test_name = name.rsplit('/', 1)[0]  # progressively strip off path segments
+            if test_name == name:
+                return None
+            return _get_channel_for_name_helper(test_name)
+
+    _stripped, platform = split_platform(channel_name, context.known_subdirs)
+    channel = _get_channel_for_name_helper(_stripped)
+
+    if channel is not None:
+        # stripping off path threw information away from channel_name (i.e. any potential subname)
+        # channel.name *should still be* channel_name
+        channel = copy(channel)
+        channel.name = _stripped
+        if platform:
+            channel.platform = platform
+        return channel
+    else:
+        ca = context.channel_alias
+        return Channel(scheme=ca.scheme, auth=ca.auth, location=ca.location, token=ca.token,
+                       name=_stripped, platform=platform)
+
+
+def _read_channel_configuration(scheme, host, port, path):
+    # return location, name, scheme, auth, token
+
+    path = path and path.rstrip('/')
+    test_url = Url(host=host, port=port, path=path).url
+
+    # Step 1. No path given; channel name is None
+    if not path:
+        return Url(host=host, port=port).url.rstrip('/'), None, scheme or None, None, None
+
+    # Step 2. migrated_custom_channels matches
+    for name, location in sorted(context.migrated_custom_channels.items(), reverse=True,
+                                 key=lambda x: len(x[0])):
+        location, _scheme, _auth, _token = split_scheme_auth_token(location)
+        if tokenized_conda_url_startswith(test_url, join_url(location, name)):
+            # translate location to new location, with new credentials
+            subname = test_url.replace(join_url(location, name), '', 1).strip('/')
+            channel_name = join_url(name, subname)
+            channel = _get_channel_for_name(channel_name)
+            return channel.location, channel_name, channel.scheme, channel.auth, channel.token
+
+    # Step 3. migrated_channel_aliases matches
+    for migrated_alias in context.migrated_channel_aliases:
+        if test_url.startswith(migrated_alias.location):
+            name = test_url.replace(migrated_alias.location, '', 1).strip('/')
+            ca = context.channel_alias
+            return ca.location, name, ca.scheme, ca.auth, ca.token
+
+    # Step 4. custom_channels matches
+    for name, channel in sorted(context.custom_channels.items(), reverse=True,
+                                key=lambda x: len(x[0])):
+        that_test_url = join_url(channel.location, channel.name)
+        if tokenized_startswith(test_url.split('/'), that_test_url.split('/')):
+            subname = test_url.replace(that_test_url, '', 1).strip('/')
+            return (channel.location, join_url(channel.name, subname), scheme,
+                    channel.auth, channel.token)
+
+    # Step 5. channel_alias match
+    ca = context.channel_alias
+    if ca.location and tokenized_startswith(test_url.split('/'), ca.location.split('/')):
+        name = test_url.replace(ca.location, '', 1).strip('/') or None
+        return ca.location, name, scheme, ca.auth, ca.token
+
+    # Step 6. not-otherwise-specified file://-type urls
+    if host is None:
+        # this should probably only happen with a file:// type url
+        assert port is None
+        location, name = test_url.rsplit('/', 1)
+        if not location:
+            location = '/'
+        _scheme, _auth, _token = 'file', None, None
+        return location, name, _scheme, _auth, _token
+
+    # Step 7. fall through to host:port as channel_location and path as channel_name
+    #  but bump the first token of paths starting with /conda for compatibility with
+    #  Anaconda Enterprise Repository software.
+    bump = None
+    path_parts = path.strip('/').split('/')
+    if path_parts and path_parts[0] == 'conda':
+        bump, path = 'conda', '/'.join(drop(1, path_parts))
+    return (Url(host=host, port=port, path=bump).url.rstrip('/'), path.strip('/') or None,
+            scheme or None, None, None)
+
+
+def parse_conda_channel_url(url):
+    (scheme, auth, token, platform, package_filename,
+     host, port, path, query) = split_conda_url_easy_parts(url, context.known_subdirs)
+
+    # recombine host, port, path to get a channel_name and channel_location
+    (channel_location, channel_name, configured_scheme, configured_auth,
+     configured_token) = _read_channel_configuration(scheme, host, port, path)
+
+    # if we came out with no channel_location or channel_name, we need to figure it out
+    # from host, port, path
+    assert channel_location is not None or channel_name is not None
+
+    return Channel(configured_scheme or 'https',
+                   auth or configured_auth,
+                   channel_location,
+                   token or configured_token,
+                   channel_name,
+                   platform,
+                   package_filename)
+
+
+# backward compatibility for conda-build
+def get_conda_build_local_url():
+    return context.local_build_root,
+
 
 def prioritize_channels(channels, with_credentials=True, subdirs=None):
     # prioritize_channels returns and OrderedDict with platform-specific channel
@@ -413,6 +485,14 @@ def prioritize_channels(channels, with_credentials=True, subdirs=None):
     return result
 
 
+def all_channel_urls(channels, subdirs=None, with_credentials=True):
+    result = IndexedSet()
+    for chn in channels:
+        channel = Channel(chn)
+        result.update(channel.urls(with_credentials, subdirs))
+    return result
+
+
 def offline_keep(url):
     return not context.offline or not is_url(url) or url.startswith('file:/')
 
